==> sqlitedbms-0.5.0.tar.gz: sqlitedbms-0.5.0/src/backend/main.c <==
#include "include/common.h"

struct config_t		config;

struct func_t		func;

/* extended sql */
regex_t			reg_admin_dump;
regex_t			reg_admin_show;
regex_t			reg_admin_desc;

/* extended sql for administrator */
regex_t			reg_admin_kill;
regex_t			reg_admin_createdb;
regex_t			reg_admin_dropdb;
regex_t			reg_admin_createuser;
regex_t			reg_admin_dropuser;
regex_t			reg_admin_alteruser;

regex_t			reg_admin_grant;
regex_t			reg_admin_revoke;
regex_t			reg_admin_grant_create;
regex_t			reg_admin_revoke_create;

struct stat_t		thread_stat;

const char		*libversion;

long long		thread_seq = 0;
pthread_mutex_t		thread_seq_mutex;

short			terminate = FALSE;

#ifdef	ENABLE_SSL
SSL_CTX			*ssl_ctx;
#endif

int main( int argc, char *argv[] )
{
	extern short	repl_slave_thread_init;

	sock_t		server_addr;
	int		server_fd;
	struct pollfd	server_poll;
	int		server_poll_res;

	sock_t		client_addr;
	socklen_t	client_addr_size;
	int		client_fd;

	pthread_t	client_thread;

	pthread_attr_t	thread_attr;

	struct passwd	*pwd;
	uid_t		uid;
	gid_t		gid;
	
	struct rlimit	limit_r;
	
	struct thread_t	*thread;
	
	int		result;

	char		admin_id[64];
	char		admin_pass[64];
	char		admin_confirm[8];
	char		*admin_tmp;
	short		admin_process;

	/* config */
	init_config();
	if ( parse_config( argc, argv ) == -1 )
	{
		//-h
		return 0;
	}

	chdir( __PREFIX );

	/* limit */
	limit_r.rlim_cur = 1024*16;
	limit_r.rlim_max = 1024*16;
	setrlimit( RLIMIT_NOFILE, &limit_r );
	setrlimit( RLIMIT_NPROC, &limit_r );

	/* sqlite lib version */
	libversion = db_version();

	/* check & load catalog */
	if ( check_catalog_exists() != TRUE )
	{
		admin_process = FALSE;
		admin_confirm[0] = '_';

		while ( admin_process == FALSE )
		{
			memset( admin_id, 0, 64 );
			memset( admin_pass, 0, 64 );

			printf( "** Catalog files not found. Insert admin id and password for initialize catalog.\n   Administrator ID: " );
			fflush( stdout );
			fgets( admin_id, 64, stdin );
			fflush( stdin );

			admin_tmp = strchr( admin_id, '\n' );
			if ( admin_tmp != NULL )
				*admin_tmp = '\0';
			admin_tmp = strchr( admin_id, '\r' );
			if ( admin_tmp != NULL )
				*admin_tmp = '\0';
			if ( strlen( admin_id ) <= 0 )
			{
				printf( "** Insert administrator ID.\n" );
				continue;
			}

			printf( "   Administrator password: " );
			fflush( stdout );
			fgets( admin_pass, 64, stdin );
			fflush( stdin );

			admin_tmp = strchr( admin_pass, '\n' );
			if ( admin_tmp != NULL )
				*admin_tmp = '\0';
			admin_tmp = strchr( admin_pass, '\r' );
			if ( admin_tmp != NULL )
				*admin_tmp = '\0';

			if ( strlen( admin_pass ) <= 0 )
			{
				printf( "** Insert administrator password.\n" );
				continue;
			}

			while ( admin_confirm[0] != 'y' || admin_confirm[0] != 'Y' || admin_confirm[0] != 'n' || admin_confirm[0] != 'N' )
			{
				printf( "   Administrator's ID: %s, password: %s\n   Right? (y/n): ", admin_id, admin_pass );
				fflush( stdout );
				fgets( admin_confirm, 8, stdin );
				fflush( stdin );
				if ( admin_confirm[0] == 'y' || admin_confirm[0] == 'Y' )
				{
					admin_process = TRUE;
					break;
				}
				else if ( admin_confirm[0] == 'n' || admin_confirm[0] == 'N' )
				{
					printf( "\n" );
					break;
				}
			}
		}
	}else{
		memset( admin_id, 0, 64 );
		memset( admin_pass, 0, 64 );
	}

	printf("** Startup SQLiteDBMS.\n");

#ifndef __DEBUG
	/* daemonize */
	pwd = getpwnam( config.server_user );
	if ( pwd == NULL )
	{
		perror( "   Unknown user" );
		exit( 1 );
	}
	uid = pwd->pw_uid;
	gid = pwd->pw_gid;

	switch ( fork() )
	{
		case 0: break;
		case -1:
			perror( "   fork() failed" );
		default:
			exit(1);
	}

	(void) setsid();

	if ( getuid() == 0 )
	{
		if ( setgroups( 0, NULL ) < 0 )
		{
			perror( "   Set groups error" );
			exit( 1 );
		}

		if ( setgid( gid ) < 0 )
		{
			perror( "   Set gid error" );
			exit( 1 );
		}

		if ( initgroups( config.server_user, gid ) < 0 )
			perror( "   Initgroups error" );

		if ( setuid( uid ) < 0 )
		{
			perror( "   Set uid error" );
			exit( 1 );
		}
	}
#endif

	printf("** Initialize Logger.\n");
	log_init();

#ifdef ENABLE_SSL
	/* init for SSL */
	if ( config.ssl == TRUE )
	{
		printf("** Initialize SSL.\n");
		SSL_library_init();
		ssl_ctx = SSL_CTX_new( SSLv23_server_method() );
		SSL_CTX_set_mode( ssl_ctx, SSL_MODE_ENABLE_PARTIAL_WRITE & SSL_MODE_AUTO_RETRY );
		result = SSL_CTX_use_certificate_file( ssl_ctx, config.ssl_cert, SSL_FILETYPE_PEM );
		if ( result != 1 )
		{
			perror( "   SSL init failed(CERT)" );
			exit( 1 );
		}

		result = SSL_CTX_use_PrivateKey_file( ssl_ctx, config.ssl_key, SSL_FILETYPE_PEM );
		if ( result != 1 )
		{
			perror( "   SSL init failed(KEY)" );
			exit( 1 );
		}

		result = SSL_CTX_check_private_key( ssl_ctx );
		if ( result != 1 )
		{
			perror( "   SSL init failed(private key check)" );
			exit( 1 );
		}
	}
#endif

	(void) signal( SIGPIPE, SIG_IGN );
	(void) signal( SIGINT, safe_exit );

	if ( auth_init( admin_id, admin_pass ) != SQLITE_OK )
		return -1;

	/* init for stat thread */
	pthread_mutex_init( &thread_seq_mutex, NULL );
	pthread_mutex_init( &thread_stat.lock, NULL );
	thread_stat.first = NULL;
	thread_stat.count = 0;

	/* query cache */
	if ( config.cache_use == TRUE )
	{
		printf("** Preparing query cache.\n");
		cache_init( config.cache_slot, config.cache_size );
	}

	/* replication master */
	if ( config.replication_master == TRUE )
	{
		printf("** Preparing replication master.\n");
		repl_master_init();
	}

	if ( config.replication_slave == TRUE )
	{
		printf("** Preparing replication slave.\n");
		repl_slave_init();

		if ( repl_slave_thread_init != TRUE )
		{
			printf("** Waiting for connect to replication master.\n");
			fflush(stdout);
			while ( repl_slave_thread_init != TRUE )
				sleep(1);
		}
	}

	/* init server socket */
	printf("** Preparing server socket.\n");
	server_fd = server_sock_init( &server_addr, config.server_port, 1000 );
	if (server_fd <= 0)
	{
		perror( "   Cannot open server socket" );
		return -1;
	}

	/* init for accept */
	server_poll.fd = server_fd;
	server_poll.events = POLLIN;

	client_addr_size = sizeof( client_addr.sa );

	/* prepare extended SQL reg */
	printf("** Preparing extended SQL.\n");

	regcomp( &reg_admin_dump,	"DUMP[ \t\n\r]+TO[ \t\n\r]+((MIRROR )(FILE) '([^']*)'|(CLIENT))", REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_show,	"SHOW[ \t\n\r]+([a-zA-Z0-9_]+)[ \t\n\r]*([a-zA-Z0-9_\"]*)", REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_desc,	"DESC[ \t\n\r]+([a-zA-Z0-9_\"]+)", REG_EXTENDED | REG_ICASE | REG_NEWLINE );

	regcomp( &reg_admin_kill,	"KILL[ \t\n\r]+([0-9]+)", REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_createdb,	"CREATE[ \t\n\r]+DATABASE ([a-zA-Z0-9_\"]+)", REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_dropdb,	"DROP[ \t\n\r]+DATABASE[ \t\n\r]+([a-zA-Z0-9_\"]+)", REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_createuser,	"CREATE[ \t\n\r]+USER[ \t\n\r]+([a-zA-Z0-9_\"]+)[ \t\n\r]+([a-zA-Z0-9_\"]+|'[^']+')[ \t\n\r]*(SUPERUSER)?", REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_dropuser,	"DROP[ \t\n\r]+USER[ \t\n\r]+([a-zA-Z0-9_\"]+)", REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_alteruser,	"ALTER[ \t\n\r]+USER[ \t\n\r]+([a-zA-Z0-9_\"]+)[ \t\n\r]+SET[ \t\n\r]+(((PASSWORD)[ \t\n\r]+([a-zA-Z0-9_\"]+|'[^']+'))|((NO)?(SUPERUSER)))", REG_EXTENDED | REG_ICASE | REG_NEWLINE );

	regcomp( &reg_admin_grant,
		"GRANT[ \t\n\r]+(ALL|SELECT|INSERT|UPDATE|DELETE|ALTER|TRIGGER|SHOW)[ \t\n\r]+ON[ \t\n\r]+([a-zA-Z0-9_\"*]+)[.]?([a-zA-Z0-9_\"*]+)?[ \t\n\r]+TO[ \t\n\r]+(([a-zA-Z0-9_\"]+)|PUBLIC)",
		REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_revoke,
		"REVOKE[ \t\n\r]+(ALL|SELECT|INSERT|UPDATE|DELETE|ALTER|TRIGGER|SHOW)[ \t\n\r]+ON[ \t\n\r]+([a-zA-Z0-9_\"*]+)[.]?([a-zA-Z0-9_\"*]+)?[ \t\n\r]+TO[ \t\n\r]+(([a-zA-Z0-9_\"]+)|PUBLIC)",
		REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_grant_create,
		"GRANT[ \t\n\r]+CREATE[ \t\n\r]+(ALL|TABLE|VIEW|INDEX|TRIGGER)[ \t\n\r]+ON[ \t\n\r]+([a-zA-Z0-9_\"*]+)[ \t\n\r]+TO[ \t\n\r]+(([a-zA-Z0-9_\"]+)|PUBLIC)",
		REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_revoke_create,
		"REVOKE[ \t\n\r]+CREATE[ \t\n\r]+(ALL|TABLE|VIEW|INDEX|TRIGGER)[ \t\n\r]+ON[ \t\n\r]+([a-zA-Z0-9_\"*]+)[ \t\n\r]+TO[ \t\n\r]+(([a-zA-Z0-9_\"]+)|PUBLIC)",
		REG_EXTENDED | REG_ICASE | REG_NEWLINE );

	/* thread attr init */
	pthread_attr_init( &thread_attr );
	pthread_attr_setdetachstate( &thread_attr, PTHREAD_CREATE_DETACHED );

	/* execute external function */
	if ( func.after_fork != NULL )
	{
		result = func.after_fork();
		if ( result == STAT_PROC_ERROR || result == STAT_PROC_GO_EXIT )
		{
			perror("   Terminate on execute after fork function");
			exit( 1 );
		}
	}

	log_server( "***** SQLiteDBMS is ready." );
	printf("** SQLiteDBMS is ready.\n");

	/* start main loop */
	while ( 1 )
	{
		server_poll_res = poll( &server_poll, 1, config.poll_timer );
		if ( server_poll_res > 0 )
		{
			client_fd = accept( server_fd, &client_addr.sa, &client_addr_size );

			if ( client_fd < 0 )
				continue;

			if ( client_sock_nonblock( client_fd ) == -1 )
			{
				close( client_fd );
				continue;
			}

			thread = malloc( sizeof(struct thread_t) );
			memset( thread, 0, sizeof(struct thread_t) );
			thread->client_fd = client_fd;
			strncpy( thread->client_ip, inet_ntoa( client_addr.sa_in.sin_addr ), 20 );

			/* execute external function */
			if ( func.after_accept != NULL )
				func.after_accept( thread );

			log_server( "**** New client accepted." );

#ifdef __DEBUG
			printf("***** New client.\n");
#endif
			if ( pthread_create( &client_thread, &thread_attr, client, thread ) != 0 )
			{
				free( thread );
				close( client_fd );
			}
		}
	}

	return 0;
}

int client_sock_nonblock( int sock )
{
	static int		client_fd_opt=1;
	static struct linger	linger = { 1, 500 };

	if ( fcntl( sock, F_SETFL, O_RDWR|O_NONBLOCK|O_ASYNC|O_NDELAY ) < 0 )
		return -1;
	setsockopt( sock, IPPROTO_TCP, TCP_NODELAY, &client_fd_opt, sizeof(client_fd_opt) );
	setsockopt( sock, SOL_SOCKET, SO_LINGER, &linger, sizeof(linger) );

	return 0;
}

short check_catalog_exists()
{
	extern struct config_t	config;
	char			filepath[1024];
	int			fd;

	snprintf( filepath, 1024, "%s/catalog.user.superdb", config.db_path );
	fd = open( filepath, O_RDONLY );
	if ( fd <= 0 )
		return FALSE;
	close( fd );
	return TRUE;
}

void safe_exit( int sig )
{
	extern struct db_t	db_auth;
	extern struct auth_t	auth;

	log_server( "***** SQLiteDBMS is shutdown.\n" );
	//sleep( 1 );

	/* execute external function */
	if ( func.on_finalize != NULL )
		func.on_finalize();

	if ( auth.user.table != NULL )
		_db_free_table( auth.user.table );
	if ( auth.db.table != NULL )
		_db_free_table( auth.db.table );
	if ( auth.object.table != NULL )
		_db_free_table( auth.object.table );
	db_close( &db_auth );

#ifdef	ENABLE_SSL
	if ( ssl_ctx != NULL )
		SSL_CTX_free( ssl_ctx );
#endif

	regfree( &reg_admin_dump );
	regfree( &reg_admin_show );
	regfree( &reg_admin_desc );
	regfree( &reg_admin_kill );
	regfree( &reg_admin_createdb );
	regfree( &reg_admin_dropdb );
	regfree( &reg_admin_createuser );
	regfree( &reg_admin_dropuser );
	regfree( &reg_admin_grant );
	regfree( &reg_admin_revoke );
	regfree( &reg_admin_grant_create );
	regfree( &reg_admin_revoke_create );

	pthread_mutex_destroy( &thread_seq_mutex );

	cache_safe_exit();
	log_safe_exit();
	repl_master_safe_exit();

	printf("** SQLiteDBMS is shutdown.\n");
	exit( 0 );
}

==> sqlitedbms-0.5.1.tar.gz: sqlitedbms-0.5.1/src/backend/main.c <==
#include "include/common.h"

struct config_t		config;

struct func_t		func;

/* extended sql */
regex_t			reg_admin_dump;
regex_t			reg_admin_show;
regex_t			reg_admin_desc;

/* extended sql for administrator */
regex_t			reg_admin_kill;
regex_t			reg_admin_createdb;
regex_t			reg_admin_dropdb;
regex_t			reg_admin_createuser;
regex_t			reg_admin_dropuser;
regex_t			reg_admin_alteruser;

regex_t			reg_admin_grant;
regex_t			reg_admin_revoke;
regex_t			reg_admin_grant_create;
regex_t			reg_admin_revoke_create;

struct stat_t		thread_stat;

const char		*libversion;

long long		thread_seq = 0;
pthread_mutex_t		thread_seq_mutex;

short			terminate = FALSE;

#ifdef	ENABLE_SSL
SSL_CTX			*ssl_ctx;
#endif

int main( int argc, char *argv[] )
{
	extern short	repl_slave_thread_init;

	sock_t		server_addr;
	int		server_fd;
	struct pollfd	server_poll;
	int		server_poll_res;

	sock_t		client_addr;
	socklen_t	client_addr_size;
	int		client_fd;

	pthread_t	client_thread;

	pthread_attr_t	thread_attr;

	struct passwd	*pwd;
	uid_t		uid;
	gid_t		gid;
	
	struct rlimit	limit_r;
	
	struct thread_t	*thread;
	
	int		result;

	char		admin_id[64];
	char		admin_pass[64];
	char		admin_confirm[8];
	char		*admin_tmp;
	short		admin_process;

	/* config */
	init_config();
	if ( parse_config( argc, argv ) == -1 )
	{
		//-h
		return 0;
	}

	chdir( __PREFIX );

	/* limit */
	limit_r.rlim_cur = RLIM_INFINITY;
	limit_r.rlim_max = RLIM_INFINITY;
	setrlimit( RLIMIT_CPU, &limit_r );
	setrlimit( RLIMIT_FSIZE, &limit_r );
	setrlimit( RLIMIT_DATA, &limit_r );
	setrlimit( RLIMIT_NOFILE, &limit_r );
	setrlimit( RLIMIT_STACK, &limit_r );
	setrlimit( RLIMIT_RSS, &limit_r );
	setrlimit( RLIMIT_NPROC, &limit_r );
	setrlimit( RLIMIT_NOFILE, &limit_r );
	setrlimit( RLIMIT_MEMLOCK, &limit_r );
	setrlimit( RLIMIT_AS, &limit_r );

	/* sqlite lib version */
	libversion = db_version();

	/* check & load catalog */
	if ( check_catalog_exists() != TRUE )
	{
		admin_process = FALSE;
		admin_confirm[0] = '_';

		while ( admin_process == FALSE )
		{
			memset( admin_id, 0, 64 );
			memset( admin_pass, 0, 64 );

			printf( "** Catalog files not found. Insert admin id and password for initialize catalog.\n   Administrator ID: " );
			fflush( stdout );
			fgets( admin_id, 64, stdin );
			fflush( stdin );

			admin_tmp = strchr( admin_id, '\n' );
			if ( admin_tmp != NULL )
				*admin_tmp = '\0';
			admin_tmp = strchr( admin_id, '\r' );
			if ( admin_tmp != NULL )
				*admin_tmp = '\0';
			if ( strlen( admin_id ) <= 0 )
			{
				printf( "** Insert administrator ID.\n" );
				continue;
			}

			printf( "   Administrator password: " );
			fflush( stdout );
			fgets( admin_pass, 64, stdin );
			fflush( stdin );

			admin_tmp = strchr( admin_pass, '\n' );
			if ( admin_tmp != NULL )
				*admin_tmp = '\0';
			admin_tmp = strchr( admin_pass, '\r' );
			if ( admin_tmp != NULL )
				*admin_tmp = '\0';

			if ( strlen( admin_pass ) <= 0 )
			{
				printf( "** Insert administrator password.\n" );
				continue;
			}

			while ( admin_confirm[0] != 'y' || admin_confirm[0] != 'Y' || admin_confirm[0] != 'n' || admin_confirm[0] != 'N' )
			{
				printf( "   Administrator's ID: %s, password: %s\n   Right? (y/n): ", admin_id, admin_pass );
				fflush( stdout );
				fgets( admin_confirm, 8, stdin );
				fflush( stdin );
				if ( admin_confirm[0] == 'y' || admin_confirm[0] == 'Y' )
				{
					admin_process = TRUE;
					break;
				}
				else if ( admin_confirm[0] == 'n' || admin_confirm[0] == 'N' )
				{
					printf( "\n" );
					break;
				}
			}
		}
	}else{
		memset( admin_id, 0, 64 );
		memset( admin_pass, 0, 64 );
	}

	printf("** Startup SQLiteDBMS.\n");

#ifndef __DEBUG
	/* daemonize */
	pwd = getpwnam( config.server_user );
	if ( pwd == NULL )
	{
		perror( "   Unknown user" );
		exit( 1 );
	}
	uid = pwd->pw_uid;
	gid = pwd->pw_gid;

	switch ( fork() )
	{
		case 0: break;
		case -1:
			perror( "   fork() failed" );
		default:
			exit(1);
	}

	(void) setsid();

	if ( getuid() == 0 )
	{
		if ( setgroups( 0, NULL ) < 0 )
		{
			perror( "   Set groups error" );
			exit( 1 );
		}

		if ( setgid( gid ) < 0 )
		{
			perror( "   Set gid error" );
			exit( 1 );
		}

		if ( initgroups( config.server_user, gid ) < 0 )
			perror( "   Initgroups error" );

		if ( setuid( uid ) < 0 )
		{
			perror( "   Set uid error" );
			exit( 1 );
		}
	}
#endif

	if ( datadir_write_check( config.db_path ) != SQLITE_OK )
	{
		perror( "** Data directory write access error" );
		printf( "   Change data directory's permision or owner.\n" );
		exit( 1 );
	}

	printf("** Initialize Logger.\n");
	if ( log_init() != SQLITE_OK )
	{
		perror( "   Logger init failed" );
		printf( "   Change log directory's permision or owner.\n" );
		exit( 1 );
	}

#ifdef ENABLE_SSL
	/* init for SSL */
	if ( config.ssl == TRUE )
	{
		printf("** Initialize SSL.\n");
		SSL_library_init();
		ssl_ctx = SSL_CTX_new( SSLv23_server_method() );
		SSL_CTX_set_mode( ssl_ctx, SSL_MODE_ENABLE_PARTIAL_WRITE & SSL_MODE_AUTO_RETRY );
		result = SSL_CTX_use_certificate_file( ssl_ctx, config.ssl_cert, SSL_FILETYPE_PEM );
		if ( result != 1 )
		{
			perror( "   SSL init failed(CERT)" );
			exit( 1 );
		}

		result = SSL_CTX_use_PrivateKey_file( ssl_ctx, config.ssl_key, SSL_FILETYPE_PEM );
		if ( result != 1 )
		{
			perror( "   SSL init failed(KEY)" );
			exit( 1 );
		}

		result = SSL_CTX_check_private_key( ssl_ctx );
		if ( result != 1 )
		{
			perror( "   SSL init failed(private key check)" );
			exit( 1 );
		}
	}
#endif

	(void) signal( SIGPIPE, SIG_IGN );
	(void) signal( SIGINT, safe_exit );

	/* db init */
	db_init();

	/* auth init */
	if ( auth_init( admin_id, admin_pass ) != SQLITE_OK )
		return -1;

	/* init for stat thread */
	pthread_mutex_init( &thread_seq_mutex, NULL );
	pthread_mutex_init( &thread_stat.lock, NULL );
	thread_stat.first = NULL;
	thread_stat.count = 0;

	/* query cache */
	if ( config.cache_use == TRUE )
	{
		printf("** Preparing query cache.\n");
		cache_init( config.cache_slot, config.cache_size );
	}

	/* replication master */
	if ( config.replication_master == TRUE )
	{
		printf("** Preparing replication master.\n");
		if ( repl_master_init() != SQLITE_OK )
		{
			perror( "   Replication master init failed" );
			printf( "   Change var/replication directory's permision or owner.\n" );
			exit( 1 );
		}
	}

	if ( config.replication_slave == TRUE )
	{
		printf("** Preparing replication slave.\n");
		repl_slave_init();

		if ( repl_slave_thread_init != TRUE )
		{
			printf("** Waiting for connect to replication master.\n");
			fflush(stdout);
			while ( repl_slave_thread_init != TRUE )
				sleep(1);
		}
	}

	/* init server socket */
	printf("** Preparing server socket.\n");
	server_fd = server_sock_init( &server_addr, config.server_port, 1000 );
	if (server_fd <= 0)
	{
		perror( "   Cannot open server socket" );
		return -1;
	}

	/* init for accept */
	server_poll.fd = server_fd;
	server_poll.events = POLLIN;

	client_addr_size = sizeof( client_addr.sa );

	/* prepare extended SQL reg */
	printf("** Preparing extended SQL.\n");

	regcomp( &reg_admin_dump,	"DUMP[ \t\n\r]+TO[ \t\n\r]+((MIRROR )(FILE) '([^']*)'|(CLIENT))", REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_show,	"SHOW[ \t\n\r]+([a-zA-Z0-9_]+)[ \t\n\r]*([a-zA-Z0-9_\"]*)", REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_desc,	"DESC[ \t\n\r]+([a-zA-Z0-9_\"]+)", REG_EXTENDED | REG_ICASE | REG_NEWLINE );

	regcomp( &reg_admin_kill,	"KILL[ \t\n\r]+([0-9]+)", REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_createdb,	"CREATE[ \t\n\r]+DATABASE ([a-zA-Z0-9_\"]+)", REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_dropdb,	"DROP[ \t\n\r]+DATABASE[ \t\n\r]+([a-zA-Z0-9_\"]+)", REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_createuser,	"CREATE[ \t\n\r]+USER[ \t\n\r]+([a-zA-Z0-9_\"]+)[ \t\n\r]+([a-zA-Z0-9_\"]+|'[^']+')[ \t\n\r]*(SUPERUSER)?", REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_dropuser,	"DROP[ \t\n\r]+USER[ \t\n\r]+([a-zA-Z0-9_\"]+)", REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_alteruser,	"ALTER[ \t\n\r]+USER[ \t\n\r]+([a-zA-Z0-9_\"]+)[ \t\n\r]+SET[ \t\n\r]+(((PASSWORD)[ \t\n\r]+([a-zA-Z0-9_\"]+|'[^']+'))|((NO)?(SUPERUSER)))", REG_EXTENDED | REG_ICASE | REG_NEWLINE );

	regcomp( &reg_admin_grant,
		"GRANT[ \t\n\r]+(ALL|SELECT|INSERT|UPDATE|DELETE|ALTER|TRIGGER|SHOW)[ \t\n\r]+ON[ \t\n\r]+([a-zA-Z0-9_\"*]+)[.]?([a-zA-Z0-9_\"*]+)?[ \t\n\r]+TO[ \t\n\r]+(([a-zA-Z0-9_\"]+)|PUBLIC)",
		REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_revoke,
		"REVOKE[ \t\n\r]+(ALL|SELECT|INSERT|UPDATE|DELETE|ALTER|TRIGGER|SHOW)[ \t\n\r]+ON[ \t\n\r]+([a-zA-Z0-9_\"*]+)[.]?([a-zA-Z0-9_\"*]+)?[ \t\n\r]+TO[ \t\n\r]+(([a-zA-Z0-9_\"]+)|PUBLIC)",
		REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_grant_create,
		"GRANT[ \t\n\r]+CREATE[ \t\n\r]+(ALL|TABLE|VIEW|INDEX|TRIGGER)[ \t\n\r]+ON[ \t\n\r]+([a-zA-Z0-9_\"*]+)[ \t\n\r]+TO[ \t\n\r]+(([a-zA-Z0-9_\"]+)|PUBLIC)",
		REG_EXTENDED | REG_ICASE | REG_NEWLINE );
	regcomp( &reg_admin_revoke_create,
		"REVOKE[ \t\n\r]+CREATE[ \t\n\r]+(ALL|TABLE|VIEW|INDEX|TRIGGER)[ \t\n\r]+ON[ \t\n\r]+([a-zA-Z0-9_\"*]+)[ \t\n\r]+TO[ \t\n\r]+(([a-zA-Z0-9_\"]+)|PUBLIC)",
		REG_EXTENDED | REG_ICASE | REG_NEWLINE );

	/* thread attr init */
	pthread_attr_init( &thread_attr );
	pthread_attr_setdetachstate( &thread_attr, PTHREAD_CREATE_DETACHED );

	/* execute external function */
	if ( func.after_fork != NULL )
	{
		result = func.after_fork();
		if ( result == STAT_PROC_ERROR || result == STAT_PROC_GO_EXIT )
		{
			perror("   Terminate on execute after fork function");
			exit( 1 );
		}
	}

	log_server( "***** SQLiteDBMS is ready." );
	printf("** SQLiteDBMS is ready.\n");

	/* start main loop */
	while ( 1 )
	{
		server_poll_res = poll( &server_poll, 1, config.poll_timer );
		if ( server_poll_res > 0 )
		{
			client_fd = accept( server_fd, &client_addr.sa, &client_addr_size );

			if ( client_fd < 0 )
				continue;

			if ( client_sock_nonblock( client_fd ) == -1 )
			{
				close( client_fd );
				continue;
			}

			thread = malloc( sizeof(struct thread_t) );
			memset( thread, 0, sizeof(struct thread_t) );
			thread->client_fd = client_fd;
			snprintf( thread->client_ip, 20, "%s", inet_ntoa( client_addr.sa_in.sin_addr ) );

			/* execute external function */
			if ( func.after_accept != NULL )
				func.after_accept( thread );

			log_server( "**** New client accepted." );

#ifdef __DEBUG
			printf("***** New client.\n");
#endif
			if ( pthread_create( &client_thread, &thread_attr, client, thread ) != 0 )
			{
				free( thread );
				close( client_fd );
			}
		}
	}

	return 0;
}

int datadir_write_check( char *db_path )
{
	FILE                    *test;
	char                    filename[512];

	snprintf( filename, 512, "%s/.init", db_path ); 
	test = fopen( filename, "w" );
	if ( test == NULL )
		return SQLITE_ERROR;
	fclose( test );
	unlink( filename );

	return SQLITE_OK;
}

int client_sock_nonblock( int sock )
{
	static int		client_fd_opt=1;
	static struct linger	linger = { 1, 500 };

	if ( fcntl( sock, F_SETFL, O_RDWR|O_NONBLOCK|O_ASYNC|O_NDELAY ) < 0 )
		return -1;
	setsockopt( sock, IPPROTO_TCP, TCP_NODELAY, &client_fd_opt, sizeof(client_fd_opt) );
	setsockopt( sock, SOL_SOCKET, SO_LINGER, &linger, sizeof(linger) );

	return 0;
}

short check_catalog_exists()
{
	extern struct config_t	config;
	char			filepath[1024];
	int			fd;

	snprintf( filepath, 1024, "%s/catalog.user.superdb", config.db_path );
	fd = open( filepath, O_RDONLY );
	if ( fd <= 0 )
		return FALSE;
	close( fd );
	return TRUE;
}

void safe_exit( int sig )
{
	extern struct db_t	db_auth;
	extern struct auth_t	auth;

	log_server( "***** SQLiteDBMS is shutdown.\n" );
	//sleep( 1 );

	/* execute external function */
	if ( func.on_finalize != NULL )
		func.on_finalize();

	if ( auth.user.table != NULL )
		_db_free_table( auth.user.table );
	if ( auth.db.table != NULL )
		_db_free_table( auth.db.table );
	if ( auth.object.table != NULL )
		_db_free_table( auth.object.table );
	db_close( &db_auth );

#ifdef	ENABLE_SSL
	if ( ssl_ctx != NULL )
		SSL_CTX_free( ssl_ctx );
#endif

	regfree( &reg_admin_dump );
	regfree( &reg_admin_show );
	regfree( &reg_admin_desc );
	regfree( &reg_admin_kill );
	regfree( &reg_admin_createdb );
	regfree( &reg_admin_dropdb );
	regfree( &reg_admin_createuser );
	regfree( &reg_admin_dropuser );
	regfree( &reg_admin_grant );
	regfree( &reg_admin_revoke );
	regfree( &reg_admin_grant_create );
	regfree( &reg_admin_revoke_create );

	pthread_mutex_destroy( &thread_seq_mutex );

	cache_safe_exit();
	log_safe_exit();
	repl_master_safe_exit();

	printf("** SQLiteDBMS is shutdown.\n");
	exit( 0 );
}
